#/*------------------------------------------------------------------------------
#--  This file is a part of the libuc, microcontroler library
#--  Copyright (C) 2011, Alexis Jeandet
#--
#--  This program is free software; you can redistribute it and/or modify
#--  it under the terms of the GNU General Public License as published by
#--  the Free Software Foundation; either version 3 of the License, or
#--  (at your option) any later version.
#--
#--  This program is distributed in the hope that it will be useful,
#--  but WITHOUT ANY WARRANTY; without even the implied warranty of
#--  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#--  GNU General Public License for more details.
#--
#--  You should have received a copy of the GNU General Public License
#--  along with this program; if not, write to the Free Software
#--  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#-------------------------------------------------------------------------------
#--                 Author : Alexis Jeandet
#--                   Mail : alexis.jeandet@gmail.com
#-------------------------------------------------------------------------------*/

#---------------------------------------------------------------------------------
#       GCC EXECUTABLES
#---------------------------------------------------------------------------------
LIBUC_PREFIX = arm-none-eabi-
LIBUC_CC  = $(LIBUC_PREFIX)gcc
LIBUC_CXX = $(LIBUC_PREFIX)g++
LIBUC_AR  = $(LIBUC_PREFIX)ar
LIBUC_AS  = $(LIBUC_PREFIX)as
LIBUC_LD  = $(LIBUC_PREFIX)ld
LIBUC_SIZE = $(LIBUC_PREFIX)size
LIBUC_STRIP = $(LIBUC_PREFIX)strip -s
LIBUC_READELF = $(LIBUC_PREFIX)readelf
LIBUC_OBJCOPY=$(LIBUC_PREFIX)objcopy
LIBUC_OBJDUMP=$(LIBUC_PREFIX)objdump



#---------------------------------------------------------------------------------
#       GCC FLAGS
#---------------------------------------------------------------------------------
LIBUC_FMCU              = -mcpu=arm7tdmi
LIBUC_CFLAGS            = $(LIBUC_FMCU)
LIBUC_CFLAGS            = -g -gdwarf-2  $(LIBUC_FMCU)
LIBUC_CFLAGS   		+= -Wall -Wextra -Wundef -Wcast-align
LIBUC_LDFLAGS  		= --gc-sections
LIBUC_ODFLAGS  		= -x --syms
LIBUC_CPFLAGS		= 
LIBUC_CFLAGS_WARN_ON	= -Wall
LIBUC_CFLAGS_WARN_OFF	= -w
LIBUC_CFLAGS_RELEASE	= -O2
LIBUC_CFLAGS_DEBUG	= -g
LIBUC_CFLAGS_RELEASE_WITH_DEBUGINFO += -O2 -g
LIBUC_CFLAGS_STATIC_LIB	= -fPIC
LIBUC_CFLAGS_SOFT_FPU	= -msoft-float
LIBUC_CFLAGS_HARD_FPU	=

LIBUC_CXXFLAGS          = $(LIBUC_FMCU)
LIBUC_CXXFLAGS         += -g -gdwarf-2 $(LIBUC_FMCU)
LIBUC_CXXFLAGS         += -Wall -Wextra -Wundef -Wcast-align
LIBUC_CXXFLAGS_WARN_ON	= -Wall
LIBUC_CXXFLAGS_WARN_OFF	= -w
LIBUC_CXXFLAGS_RELEASE	= -O2
LIBUC_CXXFLAGS_DEBUG	= -g
LIBUC_CXXFLAGS_RELEASE_WITH_DEBUGINFO += -O2 -g
LIBUC_CXXFLAGS_STATIC_LIB = -fPIC
LIBUC_CXXFLAGS_SOFT_FPU	= -msoft-float
LIBUC_CXXFLAGS_HARD_FPU	=

#---------------------------------------------------------------------------------
#       LIBRARY DEFINITIONS
#---------------------------------------------------------------------------------

LIBUC_LIBS		=
LIBUC_LIBS_CORE         = -static -lcore
LIBUC_LIBS_UCSTRINGS    = -static -lucstrings
LIBUC_LIBS_UART         = -static -luart
LIBUC_LIBS_SPI          = -static -lspi
LIBUC_LIBS_SSP          = -static -lssp
LIBUC_LIBS_IIC          = -static -liic
LIBUC_LIBS_ADC          = -static -ladc
LIBUC_LIBS_24LC0X       = -static -l24lc0X
LIBUC_LIBS_SDCARD       = -static -lsdcard
LIBUC_LIBS_BLKDEVICE    = -static -lblkdevice
LIBUC_LIBS_VS10XX       = -static -lvs10XX

LIBUC_LIBS_DIR              = $(LIBUC)/lib/bin/lpc21XX
LIBUC_LIBS_DIR_CORE         = $(LIBUC_LIBS_DIR)/CORE
LIBUC_LIBS_DIR_UCSTRINGS    = $(LIBUC_LIBS_DIR)/STRINGS/$(ARCH)
LIBUC_LIBS_DIR_UART         = $(LIBUC_LIBS_DIR)/UART
LIBUC_LIBS_DIR_SPI          = $(LIBUC_LIBS_DIR)/SPI
LIBUC_LIBS_DIR_SSP          = $(LIBUC_LIBS_DIR)/SSP
LIBUC_LIBS_DIR_IIC          = $(LIBUC_LIBS_DIR)/IIC
LIBUC_LIBS_DIR_ADC          = $(LIBUC_LIBS_DIR)/ADC
LIBUC_LIBS_DIR_24LC0X       = $(LIBUC_LIBS_DIR)/24LCXX/24LC0X
LIBUC_LIBS_DIR_SDCARD       = $(LIBUC_LIBS_DIR)/SDCARD
LIBUC_LIBS_DIR_VS10XX       = $(LIBUC_LIBS_DIR)/VS10XX
LIBUC_LIBS_DIR_BLKDEVICE    = $(LIBUC_LIBS_DIR)/BLKDEVICE

LIBUC_LIBS_DIR_CORE_CMD         = -L $(LIBUC_LIBS_DIR_CORE)
LIBUC_LIBS_DIR_UCSTRINGS_CMD    = -L $(LIBUC_LIBS_DIR_UCSTRINGS)
LIBUC_LIBS_DIR_UART_CMD         = -L $(LIBUC_LIBS_DIR_UART)
LIBUC_LIBS_DIR_SPI_CMD          = -L $(LIBUC_LIBS_DIR_SPI)
LIBUC_LIBS_DIR_SSP_CMD          = -L $(LIBUC_LIBS_DIR_SSP)
LIBUC_LIBS_DIR_IIC_CMD          = -L $(LIBUC_LIBS_DIR_IIC)
LIBUC_LIBS_DIR_ADC_CMD          = -L $(LIBUC_LIBS_DIR_ADC)
LIBUC_LIBS_DIR_24LC0X_CMD       = -L $(LIBUC_LIBS_DIR_24LC0X)
LIBUC_LIBS_DIR_SDCARD_CMD       = -L $(LIBUC_LIBS_DIR_SDCARD)
LIBUC_LIBS_DIR_VS10XX_CMD       = -L $(LIBUC_LIBS_DIR_VS10XX)
LIBUC_LIBS_DIR_BLKDEVICE_CMD    = -L $(LIBUC_LIBS_DIR_BLKDEVICE)

LIBUC_INC_DIR              = $(LIBUC)/lib/includes/lpc21XX
LIBUC_INC_DIR_CORE         = $(LIBUC_INC_DIR)/CORE
LIBUC_INC_DIR_UCSTRINGS    = $(LIBUC_INC_DIR)/STRINGS/$(ARCH)
LIBUC_INC_DIR_UART         = $(LIBUC_INC_DIR)/UART
LIBUC_INC_DIR_SPI          = $(LIBUC_INC_DIR)/SPI
LIBUC_INC_DIR_SSP          = $(LIBUC_INC_DIR)/SSP
LIBUC_INC_DIR_IIC          = $(LIBUC_INC_DIR)/IIC
LIBUC_INC_DIR_ADC          = $(LIBUC_INC_DIR)/ADC
LIBUC_INC_DIR_24LC0X       = $(LIBUC_INC_DIR)/24LCXX/24LC0X
LIBUC_INC_DIR_SDCARD       = $(LIBUC_INC_DIR)/SDCARD
LIBUC_INC_DIR_VS10XX       = $(LIBUC_INC_DIR)/VS10XX
LIBUC_INC_DIR_BLKDEVICE    = $(LIBUC_INC_DIR)/BLKDEVICE

LIBUC_INC_DIR_CORE_CMD         = -I $(LIBUC_INC_DIR_CORE)
LIBUC_INC_DIR_UCSTRINGS_CMD    = -I $(LIBUC_INC_DIR_UCSTRINGS)
LIBUC_INC_DIR_UART_CMD         = -I $(LIBUC_INC_DIR_UART)
LIBUC_INC_DIR_SPI_CMD          = -I $(LIBUC_INC_DIR_SPI)
LIBUC_INC_DIR_SSP_CMD          = -I $(LIBUC_INC_DIR_SSP)
LIBUC_INC_DIR_IIC_CMD          = -I $(LIBUC_INC_DIR_IIC)
LIBUC_INC_DIR_ADC_CMD          = -I $(LIBUC_INC_DIR_ADC)
LIBUC_INC_DIR_24LC0X_CMD       = -I $(LIBUC_INC_DIR_24LC0X)
LIBUC_INC_DIR_SDCARD_CMD       = -I $(LIBUC_INC_DIR_SDCARD)
LIBUC_INC_DIR_VS10XX_CMD       = -I $(LIBUC_INC_DIR_VS10XX)
LIBUC_INC_DIR_BLKDEVICE_CMD    = -I $(LIBUC_INC_DIR_BLKDEVICE)

#---------------------------------------------------------------------------------
#       BOARD SUPORT PACKAGES
#---------------------------------------------------------------------------------
LIBUC_BSP_DIR=$(LIBUC)/bsp
LIBUC_BSP_BIN_DIR= $(LIBUC_BSP_DIR)/bin
LIBUC_BSP_INC_DIR= $(LIBUC_BSP_DIR)/includes
LIBUC_BSP = -L $(LIBUC_BSP_BIN_DIR)/$(BSP) -static -lbsp
LIBUC_BSP_INC = -I $(LIBUC_BSP_INC_DIR)/$(BSP)
#---------------------------------------------------------------------------------
#       DEVICE SPECIAL FILES
#---------------------------------------------------------------------------------
LINKER_SCRIPT = $(ARCHFOLDER)/LPC2103_ram.ld
APPSOURCES  += $(ARCHFOLDER)/ramstartup-1.S
LPC21XX_INCDIR=$(ARCHFOLDER)


#---------------------------------------------------------------------------------
#       RULES
#---------------------------------------------------------------------------------

all: 
	@echo "lpc21XX-arm-noabi-gcc rules"


bin: $(TARGET).bin $(TARGET).hex
	@echo "compile bin"



lib: $(TARGET).a
	@echo "compile lib"

%.a: $(OBJECTFILES)
	mkdir -p $(TARGETINSTALLPATH)
	mkdir -p $(HEADERSINSTALLPATH)
	$(LIBUC_AR)  rcs $(TARGETINSTALLPATH)/$@  $(OBJECTFILES)
	cp -f $(HEADERS) $(HEADERSINSTALLPATH)/
	
%.o: %.S
	$(LIBUC_CC) -O3 -c -o $@ $< $(LIBUC_CFLAGS)

%.o: %.c
	$(LIBUC_CC) -O3 -I $(LPC21XX_INCDIR) $(LIBUC_INCLUDES) $(LIBUC_BSP_INC) -MM $< -MF $*.d -MP
	$(LIBUC_CC) -O3 -I $(LPC21XX_INCDIR) $(LIBUC_INCLUDES) $(LIBUC_BSP_INC) -c $(LIBUC_CFLAGS) $< -o $@


%.elf: $(LINKER_SCRIPT) $(OBJECTFILES)
	$(LIBUC_CC) -O3 -nostartfiles -T $^ -o $@  -static $(LIBUC_BSP) $(LIBUC_LIBRARIES) 
	$(LIBUC_OBJDUMP) $(LIBUC_ODFLAGS) $@ > $(@:.elf=.dump)
	$(LIBUC_SIZE) $@
	
%.bin: %.elf
	$(LIBUC_OBJCOPY) $(LIBUC_CPFLAGS) -O binary $< $*.bin


%.hex: %.elf
	$(LIBUC_OBJCOPY) $(LIBUC_CPFLAGS) -O ihex $< $*.hex


clean:
	rm -f *.o *.d *.bin *.hex *.dump *.map *.a
	
distclean:
	rm -f $(TARGETINSTALLPATH)/*.bin
	rm -f $(TARGETINSTALLPATH)/*.a
	








